import {
  csvParse,
  select,
  zoom,
  geoAlbersUsa,
  selectAll,
} from 'd3';
import topojson from 'topojson-client';
import { map } from './map';
import { leaflet } from './leaflet';

const USAAtlasURL =
  'https://unpkg.com/us-atlas@3.0.1/states-10m.json';

const citiesURL =
  'https://raw.githubusercontent.com/vspatel927/CS573-Final-Project/main/parks.csv';

export const viz = (container, { state, setState }) => {
  const width = window.innerWidth;
  const height = window.innerHeight;

  const svg = select(container)
    .selectAll('svg')
    .data([null])
    .join('svg')
    .attr('width', width)
    .attr('height', height);

  const tooltip = select('body')
    .append('div')
    .attr('class', 'tooltip')
    .style('opacity', 0);

  // state.data could be:
  // * undefined
  // * 'LOADING'
  // * An array of objects
  const { country, cities, toggleState } = state;

  if (country && country !== 'LOADING') {
    svg.call(map, {
      country,
      cities,
      toggleState,
    });
  }

  // if (country && country !== 'LOADING') {
  //   svg.call(leaflet, {
  //     country,
  //     cities,
  //     toggleState,
  //     svg,
  //   });
  // }

  if (country === undefined) {
    setState((state) => ({
      ...state,
      country: 'LOADING',
    }));
    fetch(USAAtlasURL)
      .then((response) => response.json())
      .then((topoJSONData) => {
        const country = topojson.feature(
          topoJSONData,
          'states',
        );
        setState((state) => ({
          ...state,
          country,
        }));
      });
    return;
  }

  if (cities === undefined) {
    setState((state) => ({
      ...state,
      cities: 'LOADING',
    }));
    fetch(citiesURL)
      .then((response) => response.text())
      .then((csvString) => {
        const cities = csvParse(csvString, (d) => {
          d['2022'] = +d['2022'].replaceAll(',', '');
          d['2023'] = +d['2023'].replaceAll(',', '');
          d.lat = +d.lat;
          d.lng = +d.lng;
          return d;
        });

        setState((state) => ({
          ...state,
          cities,
        }));
      });
    return;
  }

  if (toggleState === undefined) {
    setState((state) => ({
      ...state,
      toggleState: false,
    }));
    return;
  }

  //Zooming + panning
  var zoom = d3
    .zoom()
    .scaleExtent([1, 15])
    .on('zoom', zoomed);

  svg.call(zoom);

  function zoomed(event) {
    svg
      .selectAll('path')
      .attr('transform', event.transform);

    svg
      .selectAll('.park')
      .attr('transform', event.transform)
      .attr('r', 5 / event.transform.k);
  }

  const max = d3.max(cities, function (d) {
    return +d['2023'];
  });

  // Define the legend dimensions and position
  const legendWidth = 20;
  const legendHeight = 200;
  const legendX = 10;
  const legendY = 10;

  // Create a linear color scale for the legend
  const legendColorScale = d3
    .scaleLinear()
    .domain([0, legendHeight])
    .range(['#6F4930', '#ffba00']);

  const toolBar = svg
    .append('rect')
    .attr('x', width - 45)
    .attr('y', height - 45)
    .attr('class', 'toolBar')
    .attr('width', 50)
    .attr('height', 50)
    .attr('stroke', 'black')
    .attr('fill', 'white');

  var icon = svg
    .append('image')
    .attr(
      'xlink:href',
      'https://cdn-icons-png.flaticon.com/512/1085/1085738.png',
    )
    .attr('height', '40px')
    .attr('width', '40px')
    .attr('x', width - 42)
    .attr('y', height - 43)
    .attr('opacity', toggleState ? 0.4 : 1);

  // Append legend rectangle
  var legendRect = svg
    .append('rect')
    .attr('x', legendX)
    .attr('y', legendY)
    .attr('class', 'legendRect')
    .attr('width', legendWidth)
    .attr('height', legendHeight)
    .style('fill', 'url(#gradient)');

  //Gradient for legend
  var gradient = svg
    .append('linearGradient')
    .attr('id', 'gradient')
    .attr('gradientUnits', 'userSpaceOnUse')
    .attr('x1', 0)
    .attr('y1', legendY)
    .attr('x2', 0)
    .attr('y2', legendY + legendHeight)
    .selectAll('stop')
    .data(d3.ticks(0, 1, 10))
    .enter()
    .append('stop')
    .attr('offset', (d) => `${d * 100}%`)
    .attr('stop-color', (d) =>
      legendColorScale(d * legendHeight),
    );

  // Append legend labels
  const legendScale = d3
    .scaleLinear()
    .domain([0, max])
    .range([legendY + legendHeight, legendY]);

  const legendAxis = d3.axisRight(legendScale).ticks(5);

  var legendTicks = svg
    .append('g')
    .attr(
      'transform',
      `translate(${legendX + legendWidth}, 0)`,
    )
    .call(legendAxis)
    .select('.domain')
    .remove();

  if (!toggleState) {
    selectAll('legendRect').join(
      (enter) => legendRect,
      (exit) => exit.remove(),
    );

    selectAll('linearGradient').join(
      (enter) => gradient,
      (exit) => exit.remove(),
    );

    selectAll('g').join(
      (enter) => legendTicks,
      (exit) => exit.remove(),
    );
  }

  icon.on('click', () => {
    setState((state) => ({
      ...state,
      toggleState: !state.toggleState,
    }));
  });

  icon
    .on('mouseover', function (event, d) {
      select(this).attr('opacity', 0.7);
      tooltip
        .transition()
        .duration(1000)
        .style('opacity', 0.9);
      tooltip
        .html(function () {
          return `
      <div style="font-family: Arial, sans-serif; font-size: 14px; padding: 8px; background-color: rgba(255, 255, 255, 0.9); color: #333; border-radius: 5px; box-shadow: 0 2px 4px rgba(0, 0, 0, 0.1);">
        <strong>Toggle Visitor Color Scale <br>
      </div>`;
        })
        .style('position', 'fixed')
        .style('pointer-events', 'none');

      tooltip
        .style('left', window.innerWidth - 100 - 20 + 'px')
        .style('top', window.innerHeight - 80 - 20 + 'px');
    })
    // Hide tooltip on mouseout
    .on('mouseout', function () {
      select(this).attr('opacity', toggleState ? 0.4 : 1);
      tooltip.interrupt().style('opacity', 0);
    });
};
